#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

#define F_CPU 8000000UL

#include <avr/delay.h>

#include <stdio.h>

#define SetBit(port,bit) port |=  _BV(bit)
#define ClrBit(port,bit) port &= ~_BV(bit)


#define Port4Out(port,bit) port |=  _BV(bit)
#define Port4Inp(port,bit) port &= ~_BV(bit)

#define HiBYTE(w) (uint8_t)((w >> 8) & 0x00ff)
#define LoBYTE(w) (uint8_t)((w >> 0) & 0x00ff)

#define Nop() __asm__ __volatile__ ("nop" ::)


//#define RCVPCH1 37300
#define RCVPCH1 38500
#define RCVPCH2 10700

#define ADR_EE_RC5    0x01A0  // 18* 2 Bytes
#define ADR_EE_vMax   0x01F0  // 1 * 1 Bytes
#define ADR_EE_vMin   0x01F1  // 1 * 1 Bytes
#define ADR_EE_vWrk   0x01F2  // 1 * 1 Bytes
#define ADR_EE_Lo     0x01F8  // 1 * 4 Bytes
#define ADR_EE_Hi     0x01FC  // 1 * 4 Bytes

#define VolumeMAX   110
#define VolumeMIN    50


#define LM7001_Port_ini   DDRC
#define LM7001_Port      PORTC
#define LM7001_BitClc      PC2
#define LM7001_BitDat      PC1
#define LM7001_BitCEN      PC3


#define I2C_Port_ini   DDRC
#define I2C_Port      PORTC
#define I2C_BitClc      PC5
#define I2C_BitDat      PC4

#define KBD_Port1_ini   DDRB
#define KBD_Port2_ini   DDRD
#define KBD_Port1_Inp   PINB
#define KBD_Port2_Inp   PIND
#define KBD_Port1      PORTB
#define KBD_Port2      PORTD
#define KBD_BitInp1_1    PB0
#define KBD_BitInp1_2    PB1
#define KBD_BitInp2_1    PD2
#define KBD_BitInp2_2    PD5
#define KBD_BitInp2_3    PD6
#define KBD_BitInp2_4    PD7

#define KBD_0    0
#define KBD_1    1
#define KBD_2    2
#define KBD_3    3
#define KBD_4    4
#define KBD_5    5
#define KBD_6    6
#define KBD_7    7
#define KBD_8    8
#define KBD_9    9
#define KBD_F    10
#define KBD_Clr  11
#define KBD_Pls  12
#define KBD_Mns  13
#define KBD_Sql  14
#define KBD_Vol  15

#define IND_0     4
#define IND_1     0
#define IND_2     3
#define IND_3     5
#define IND_4     8
#define IND_5    11
#define IND_6    13
#define IND_7     9
#define IND_8    12
#define IND_9    14
#define IND_F     1
#define IND_Clr   6
#define IND_Pls  10
#define IND_Mns  15
#define IND_Sql   2
#define IND_Vol   7

uint8_t Ind2KBD[16]={KBD_1,KBD_F,KBD_Sql,KBD_2,
		     KBD_0,KBD_3,KBD_Clr,KBD_Vol,
		     KBD_4,KBD_7,KBD_Pls,KBD_5,
		     KBD_8,KBD_6,KBD_9,  KBD_Mns};

#define WFM  0
#define NFM  1
#define  AM  2

enum { UP, DOWN };

enum { RADIO,
       STEP,
       GAIN,
       UAPY,
       FINPUT,
       WR_H_L,
       WR_MEM,
       RC_MEM,
       SCAN_F,
       SCAN_M,
       SET_SQL,
       SET_VOL
     };

#define  NUM_MEM 100

#define  InpBufLEN 7

#define  RxInpBufLEN 16

#define  PROMPT "\n>"
#define  ERROR  "\nErr\n>"

volatile uint8_t  contrast = 60;

volatile uint8_t RcInpBuf[RxInpBufLEN];
volatile uint8_t RxInpBuf[RxInpBufLEN];
volatile uint8_t RxInd = 0;
volatile uint8_t TxInd = 0;
volatile uint8_t RcInd = 0;

volatile uint8_t EchoYes = 1;

volatile uint8_t InpBuf[InpBufLEN];
	 uint8_t cccBuf[8];

volatile uint8_t InpInd;
volatile int8_t  IndMEM = 0;

	 uint8_t Step[8]={5,10,15,20,25,30,50,100};

#define  WHIDE    128

volatile uint16_t Fscope;
volatile uint8_t  Dscope;
volatile uint8_t  Iscope =  0;
volatile uint8_t  Cursor = 64;
volatile uint8_t  StrPos =  0;

volatile uint32_t FrcvWork;
volatile uint8_t  FrcvStep = 10;
volatile uint8_t  FindStep =  1;

volatile uint8_t  direction = UP;
volatile uint8_t  Regim;

volatile uint8_t  Modulation;

volatile uint8_t  FlagMute;
volatile uint8_t  FlagSQL = 1;
volatile uint16_t MuteCnt = 0;

volatile uint8_t Pwm1;
volatile uint8_t Pwm2;

volatile uint8_t Vol_Max;
volatile uint8_t Vol_Min;


#define  AUTO_CLR     7  // sec

volatile uint8_t AutoClr;
volatile uint8_t TimeClr;
volatile uint8_t TimeTmr;


#define  MUTE     50

#define  RCV_MAX  855000
#define  RCV_MIN   45000


volatile uint32_t FrcvMax;
volatile uint32_t FrcvMin;

	 uint8_t  FlagScanF = 0;
	 uint8_t  FlagScanM = 0;
	 uint8_t  FlagBegin = 0;

	  int8_t  ScanMBank = 0;
	 uint8_t  FlagBank;


volatile uint16_t FrcvPCH2;
volatile uint8_t  flagBo;

volatile uint8_t  Smeter;

volatile uint8_t  TunerGain  = 0;


static  void KeyNop  (void);
	void KeyMOD  (void);
	void KeyPlus (void);
	void KeyMinus(void);
	void KeyRADIO(void);
	void KeyGAIN (void);
	void KeyUapy (void);
	void KeyFinp (void);
	void KeySetHi(void);
	void KeySetLo(void);
	void KeyWrMem(void);
	void KeyRcMem(void);
	void KeyScanF(void);
	void KeyScanM(void);

	void KeySQL(void);
	void KeyVOL(void);

	void Key_0   (void);
	void Key_1   (void);
	void Key_2   (void);
	void Key_3   (void);
	void Key_4   (void);
	void Key_5   (void);
	void Key_6   (void);
	void Key_7   (void);
	void Key_8   (void);
	void Key_9   (void);

void (* SwKey[8])(void) =
     {
	KeyNop,
	Key_1,
	Key_2,
	Key_3,
	Key_4,
	Key_5,
	Key_6,
	KeyNop
     };


#define MT10T_Comma 0x10

#define MT10T_P     0b10101011
#define MT10T_S     0b11001101
#define MT10T_E     0b10001111
#define MT10T_B     0b11000111
#define MT10T_O     0b11101110
#define MT10T_H     0b11100011
#define MT10T_L     0b10000110
#define MT10T_U     0b11100110
#define MT10T_G     0b11001110
#define MT10T_F     0b10001011
#define MT10T_A     0b11101011
#define MT10T_N     0b01000011
#define MT10T_T     0b10000111
#define MT10T_I     0b01100000
#define MT10T_Y     0b11100101

#define MT10T_c     0b00000111
#define MT10T_q     0b11101001

#define MT10T__     0b00000100
#define MT10T_Poisk 0b11101010
#define MT10T_Minus 0b00000001


#define Lcd_Comma 0x00
#define Lcd_P     'P'
#define Lcd_S     'S'
#define Lcd_T     'T'
#define Lcd_E     'E'
#define Lcd_N     'N'
#define Lcd_G     'G'
#define Lcd_A     'A'
#define Lcd_B     'B'
#define Lcd_I     'I'
#define Lcd_F     'F'
#define Lcd_O     'O'
#define Lcd_U     'U'
#define Lcd_H     'H'
#define Lcd_L     'L'
#define Lcd_Y     'Y'
#define Lcd_c     'c'
#define Lcd_q     'q'
#define Lcd__     '_'
#define Lcd_Poisk 0xA8
#define Lcd_Minus '-'


#define Lcd4i     5
#define Lcd4f     3

	void TIC32ini (uint8_t voltave);
	void TIC32clr (uint8_t  c);
	void TIC32Pos (uint8_t  y);
	void TIC32Line(uint8_t *p);

	void TIC32ContrastP(void);
	void TIC32ContrastM(void);

	void TIC32Prn(uint8_t *p);
	void TIC32Cur(uint8_t x,uint8_t sw);

	void ScopeMeter(void);

	void Scope2Lcd(void);

	void    cMEM(void);
	void    wMEM(void);
	uint8_t rMEM(void);

	void VolumeUp  (void);
	void VolumeDown(void);

	void uart_putstr(char *p);
	int  uart_putchar(char c);

	void wBufEEPROM(uint16_t addr, uint8_t *buff, uint8_t size);
	void rBufEEPROM(uint16_t addr, uint8_t *buff, uint8_t size);

	void SQLup  (void);
	void SQLdown(void);

	void UapyUp  (void);
	void UapyDown(void);

	  uint8_t * ul2a(uint32_t c,uint8_t len);
	  uint32_t  a2l(uint8_t *p);

/////////////////////////////////////////////////
void SetFmax(void) { FrcvMax = RCV_MAX; return; }
void SetFmin(void) { FrcvMin = RCV_MIN; return; }

/////////////////
void KeyNop(void)
{
  return;
}


///////////////////
////// U-apy //////
///////////////////

#define UapyNN  10
#define UapyHi  235
#define UapyLo  210

volatile uint8_t  Uapy = 0xff;
volatile uint8_t  UapyFlg = 1;
	 uint8_t  UapySet = 0xff;
	 uint8_t  UapyCnt;
	 uint16_t SmeterUapy = 0;

//////////////////
void UapyOut(void)
{
     OCR1B = Pwm2 = Uapy;
}

/////////////////////////////////////
void BusPause(void) { _delay_us(5); }


volatile uint8_t  LcdWork = 1;
volatile uint8_t  LcdBuf [32];
volatile uint8_t  BufBuf [32];

volatile uint8_t  LcdYes;
volatile uint8_t  LcdCnt;
volatile uint8_t  LcdFlg;
volatile uint16_t LcdBlink;

/////////////////
void LcdTmr(void)
{
   if(++LcdCnt > 50)
   {
	LcdCnt = 0;
	LcdYes++;

     if(LcdBlink)
     {
	  LcdFlg = (LcdFlg)? 0:1;
     }
     else LcdFlg = 0;
   }
   return;
}

volatile uint8_t  KbdKbd = 0;
volatile uint8_t  KbdYes = 0;
volatile uint8_t  KbdCnt;
volatile uint8_t  KbdOld;

volatile uint8_t  TmrInd;

/////////////////
void KbdWrk(void)
{
 uint8_t KbdInd;

    if(KbdCnt < 255)
       KbdCnt++;

    KbdInd = 0;

    if( bit_is_clear(KBD_Port1_Inp,KBD_BitInp1_1) ) KbdInd = 1;
    if( bit_is_clear(KBD_Port1_Inp,KBD_BitInp1_2) ) KbdInd = 2;

    if( bit_is_clear(KBD_Port2_Inp,KBD_BitInp2_1) ) KbdInd = 3;
    if( bit_is_clear(KBD_Port2_Inp,KBD_BitInp2_2) ) KbdInd = 4;
    if( bit_is_clear(KBD_Port2_Inp,KBD_BitInp2_3) ) KbdInd = 5;
    if( bit_is_clear(KBD_Port2_Inp,KBD_BitInp2_4) ) KbdInd = 6;

    if( KbdInd )
    {
	 if(KbdYes && KbdOld == KbdInd)
	 {
	    //if( KbdInd == IND_Pls ||
	    //    KbdInd == IND_Mns )
	    //{
	      if(KbdCnt > 128)
	      {
		 //KbdKbd = Ind2KBD[KbdInd]+1;
		 KbdKbd = KbdInd;
	      }
	    //}
	 }
	 else
	 {
	    KbdYes = 1;
	    KbdCnt = 0;
	    KbdOld = KbdInd;
	    //KbdKbd = Ind2KBD[KbdInd]+1;
	    KbdKbd = KbdInd;
	 }
    }
    else
    {
       KbdYes = 0;
    }
}


////////////////////
uint8_t GetKbd(void)
{
  uint8_t tmp;

      //cli();
      tmp = KbdKbd;

   if(tmp)
   {
      KbdKbd = 0;
   }
   //sei();
   return(tmp);
}


/////////////////////////////
/////////////////////////////
     volatile  int8_t TMRxxx;

void TmrYYY(void)
{
  if(TMRxxx)
  {
     TMRxxx = 0;

     KbdWrk();
  }
  return;
}

/////////////////////////
// Interrupt by Timer0 //
/////////////////////////
//
// Start by 64 mks
//
SIGNAL (SIG_OVERFLOW0)
{
  if(TmrInd <  254)
     TmrInd++;

  if(TmrInd ==  64)
  {
     TmrInd = 0;
     TMRxxx++;

     LcdTmr();

     if(MuteCnt < MUTE)
	MuteCnt++;

     if(++TimeTmr == 0)   // 1048 ms
     {
       if(TimeClr)
       {
	  if(--TimeClr ==  0) AutoClr++;
       }
     }
  }
}

////////////////////////////
// Interrupt by Rx RS-232 //
////////////////////////////
SIGNAL (SIG_UART_RECV)
{
  uint8_t tmp;

  if(RxInd < RxInpBufLEN)
  {
     RxInpBuf[RxInd++] = UDR;
  }
  else tmp = UDR;
}

//--------------//
void ioinit (void)
{
    // Init PORTs

    Port4Inp(KBD_Port1_ini,KBD_BitInp1_1); // KBD inp1 -> Input
    Port4Inp(KBD_Port1_ini,KBD_BitInp1_2); // KBD inp2 -> Input

    Port4Inp(KBD_Port2_ini,KBD_BitInp2_1); // KBD inp3 -> Input
    Port4Inp(KBD_Port2_ini,KBD_BitInp2_2); // KBD inp4 -> Input
    Port4Inp(KBD_Port2_ini,KBD_BitInp2_3); // KBD inp5 -> Input
    Port4Inp(KBD_Port2_ini,KBD_BitInp2_4); // KBD inp6 -> Input

    SetBit  (KBD_Port1,KBD_BitInp1_1);   //     Input (+Ucc --> "1")
    SetBit  (KBD_Port1,KBD_BitInp1_2);   //     Input (+Ucc --> "1")

    SetBit  (KBD_Port2,KBD_BitInp2_1);   //     Input (+Ucc --> "1")
    SetBit  (KBD_Port2,KBD_BitInp2_2);   //     Input (+Ucc --> "1")
    SetBit  (KBD_Port2,KBD_BitInp2_3);   //     Input (+Ucc --> "1")
    SetBit  (KBD_Port2,KBD_BitInp2_4);   //     Input (+Ucc --> "1")

    Port4Out(LM7001_Port_ini,LM7001_BitCEN);  // Lm7001 Enable
    Port4Out(LM7001_Port_ini,LM7001_BitClc);  // Lm7001 Clock
    Port4Out(LM7001_Port_ini,LM7001_BitDat);  // Lm7001 Data

    Port4Out(I2C_Port_ini,I2C_BitClc); // I2C clc -> Out
    Port4Out(I2C_Port_ini,I2C_BitDat); // I2C dat -> Out

    SetBit(I2C_Port,I2C_BitClc);  // Init I2C
    SetBit(I2C_Port,I2C_BitDat);


    // Init ADC

    Port4Inp(DDRC,PC0);

    ADMUX  = 0x00;
    ADCSRA = 0x85; // 1000 0101  ADC on, interrupt disable, MCU clock / 32


    // Init Timer0

    TCCR0  = 0x01;       // Timer0 running on MCU clock / 1
    SetBit(TIMSK,TOIE0); // enable interrupt from Timer0


    // Init Timer1

  //Port4Out(DDRB,PB1); // output PWM1 (PB1/OC1A)
    Port4Out(DDRB,PB2); // output PWM2 (PB2/OC1B)

  //TCCR1A = 0B10100001; // Timer1 is duble 8-bit PWM
    TCCR1A = 0B00100001; // Timer1 is duble 8-bit PWM
    TCCR1B = 0B00000001; // Timer1 running on MCU clock / 1

  //OCR1A = Pwm1 = 255;  // set PWM1 value
    OCR1B = Pwm2 = 255;  // set PWM2 value

    // ### RS-232 ###

    Port4Inp(DDRD,PD0); // RS-232 Rx (Inp)
    Port4Out(DDRD,PD1); // RS-232 Tx (Out)

    UBRRL = (F_CPU / (16 * 9600UL)) - 1;        // Bd 9600
    UCSRB = _BV(TXEN) | _BV(RXEN) | _BV(RXCIE); // Tx,Rx,Rx interrupt enable

    // enable interrupts
       sei ();
       return;
}

///////////////////////////////////////
void Pause200(void) { _delay_us(200); }

/////////////////////
void sleep(int16_t s)
{
  for(s*=5; s > 0; s--)
  {
  TmrYYY();
      Pause200();
  }
}


////////////////////////
int uart_putchar(char c)
{
  if (c == '\n')
     uart_putchar('\r');

  //loop_until_bit_is_set(UCSRA, UDRE);

  while(bit_is_clear(UCSRA, UDRE))
  {
     Nop();
  }
  UDR  = c;
  return 0;
}

////////////////////////
void uart_putstr(char *p)
{
   while(*p)
   {
     uart_putchar(*p++);
   }
   return;
}

////////////////////////

void PutHEX(uint8_t c)
{
  static char hex[16]= {'0','1','2','3','4','5','6','7',
			'8','9','A','B','C','D','E','F'};

 uart_putchar('[');

   uart_putchar(hex[c >>   4]);
   uart_putchar(hex[c & 0x0f]);

 uart_putchar(']');
   return;
}


#define CH_SMETR  0x40
#define CH_GND    0x4f

#define CH_VOLUME  0x42
#define CH_JOYSTIC 0x41


//--------------//
uint16_t GetADC (uint8_t chanel)
{
  ADMUX = chanel;

  Pause200();
  SetBit(ADCSRA,ADSC); // start ADC

  while(bit_is_set(ADCSRA,ADSC))
  {
    TmrYYY();
     //Nop();
  }

  Pause200();
  SetBit(ADCSRA,ADSC); // start ADC

  while(bit_is_set(ADCSRA,ADSC))
  {
    TmrYYY();
     //Nop();
  }
  return(ADCW);
}



////////////
void wEEPROM(uint16_t addr, uint8_t byte)
{
     while(bit_is_set(EECR,EEWE))
     {
       Nop();
     }
     EEAR = addr;
     EEDR = byte;
     cli ();
	 SetBit(EECR,EEMWE);
	 SetBit(EECR,EEWE ); // start write to eeprom
     sei ();
     return;
}

///////////////
uint8_t rEEPROM(uint16_t addr)
{
     while(bit_is_set(EECR,EEWE))
     {
       Nop();
     }
     EEAR = addr;
     SetBit(EECR,EERE);
     return(EEDR);
}

///////////////
void wBufEEPROM(uint16_t addr, uint8_t *buff, uint8_t size)
{
     uint8_t i;

     for(i=0; i < size; i++)
     {
       wEEPROM(addr++,*buff); buff++;
     }
     return;
}

///////////////
void rBufEEPROM(uint16_t addr, uint8_t *buff, uint8_t size)
{
     uint8_t i;

     for(i=0; i < size; i++)
     {
	*buff = rEEPROM(addr++);
	 buff++;
     }
     return;
}

/////////////////////////
void SetFwork(uint32_t f)
{
     if( f <=  FrcvMax &&
	 f >=  FrcvMin )
     {
	 FrcvWork = (f/5)*5;
     }
     KeyRADIO();
}

/////////////////////////
void Lm7001Out(uint8_t c)
{
  uint8_t i;

    for(i=0; i < 8; i++)
    {
      if(c & 0x01)
	 SetBit(LM7001_Port,LM7001_BitDat);
      else
	 ClrBit(LM7001_Port,LM7001_BitDat);

	BusPause();
	 SetBit(LM7001_Port,LM7001_BitClc);
	BusPause();
	 ClrBit(LM7001_Port,LM7001_BitClc);

      c >>= 1;
    }
    return;
}


////////////////////
void Lm7001Set(void)
{
  static uint8_t  Mod4Lm7001[3] = {0xE6,0xE4,0xE2};
	 uint16_t Gscope;

     Fscope = (RCVPCH1-RCVPCH2)+10*((int16_t)Iscope-WHIDE/2);
     Gscope = Fscope/5;

     cli ();
     SetBit(LM7001_Port,LM7001_BitCEN);


     Lm7001Out(LoBYTE(Gscope));
     Lm7001Out(HiBYTE(Gscope));

     flagBo = Mod4Lm7001[Modulation];

	  ///////////////////////////////////////
	  //
	  //  S R2 R1 R0 TB Bo3 Bo2 Bo1
	  //
	  //  R0=0, R1=1, R2=1 --> 5kHz
	  //  R0=1, R1=1, R2=0 --> 1kHz
	  //   S=1  --> FMIN
	  //   S=0  --> AMIN
	  //
	  //      flagBo = 0xE0;  // 5kHz, FMIN
	  //      flagBo = 0xB0;  // 1kHz, FMIN  (Max 16383mHz)
	  //
	  // Bo1  --> 1 - Mute ON 0 - Mute OFF
	  // Bo2  --> 1 - WFM  0 - NFM
	  // Bo3  --> 1 - FM   0 -  AM

     if(FlagMute) { flagBo |= 0x01; }

     Lm7001Out(flagBo);

     ClrBit(LM7001_Port,LM7001_BitCEN);
     ClrBit(LM7001_Port,LM7001_BitDat);

     sei ();
     return;
}

//////////////////////////////////////
void I2CPause(void)
{
   _delay_us(1); // 10
}

///////////////////
void I2CStart(void)
{
     SetBit(I2C_Port,I2C_BitClc);
     SetBit(I2C_Port,I2C_BitDat);

    I2CPause();
     ClrBit(I2C_Port,I2C_BitDat);

    I2CPause();
     ClrBit(I2C_Port,I2C_BitClc);
}

//////////////////
void I2CStop(void)
{
     ClrBit(I2C_Port,I2C_BitDat);
    I2CPause();
     SetBit(I2C_Port,I2C_BitClc);

    I2CPause();
     SetBit(I2C_Port,I2C_BitDat);
    I2CPause();
}

//////////////////
void I2CSync(void)
{
    //I2CPause();
    //_delay_us(1);
     SetBit(I2C_Port,I2C_BitClc);

    //I2CPause();
    //_delay_us(1);
    Nop();
     ClrBit(I2C_Port,I2C_BitClc);
    //_delay_us(1);
}

//////////////////////////
void I2COutByte(uint8_t c)
{
  uint8_t i;

  cli();

    for(i=0; i < 8; i++)
    {

      if(c & 0x80)
	 SetBit(I2C_Port,I2C_BitDat);
      else
	 ClrBit(I2C_Port,I2C_BitDat);

	I2CSync();

	c <<= 1;
    }
    ClrBit(I2C_Port,I2C_BitDat);

    I2CSync(); // ASK

  sei();

    return;
}


//////////////////
void fWorkPrn(void)
{
  uint8_t i;

     uart_putstr("\nS ");

     GetADC(CH_SMETR);

     for(i = 0; i < 6; i++)
     {
	uart_putchar(cccBuf[i]);

	if(i == 2)
	      uart_putchar('.');
     }
     return;
}

//////////////////
void SetFrcv(void)
{
     Lm7001Set();
     fWorkPrn();
     return;
}

//////////////////
void ScopeUpDown(void)
{
    switch(direction)
    {
      case UP:   if(Iscope <  WHIDE-1)
		 {
		    Iscope++;
		 }
		 else
		    direction = DOWN;

		 break;

      case DOWN: if(Iscope > 0)
		 {
		    Iscope--;
		 }
		 else
		    direction = UP;

		 break;
    }
    return;
}


uint8_t disp[4] = {0x01,0x00,0x00,0x00};

//////////////////////////////////
void Vline(uint8_t x1, uint8_t x2)
{
  uint8_t   i;

     if(x1 > 31) x1 = 31;
     if(x2 > 31) x2 = 31;

     if(x1 > x2)
     {
	 i = x2;
	x2 = x1;
	x1 = i;
     }

     for(i=0; i<32;i++)
     {
	if(i < x1 || i > x2)
	{
	  disp[i/8] &= ~_BV(i%8);
	}
	else
	  disp[i/8] |=  _BV(i%8);
     }
}

////////////////////
void SetCursor(void)
{
   if(Cursor == Dscope)
   {
     disp[0] &= 0xf0;
     disp[0] |= 0x07;
   }
   if( Cursor+1 == Dscope ||
       Cursor-1 == Dscope )
   {
     disp[0] &= 0xf0;
     disp[0] |= 0x01;
   }
}

//////////////////
void ScanerS(void)
{
     Dscope = Iscope;
     SetCursor();
     ScopeUpDown();

     Lm7001Set();

    TIC32Pos (127-Dscope);
    TIC32Line(disp);

     sleep(4);

     ScopeMeter();
}

      void RsControl(void);


// --------- //
int main (void)
{
  uint8_t key;

    ioinit ();

    SetFmax();
    SetFmin();

    rBufEEPROM (ADR_EE_vMax, (uint8_t *)&Vol_Max, 1);
    rBufEEPROM (ADR_EE_vMin, (uint8_t *)&Vol_Min, 1);

    rBufEEPROM (ADR_EE_vWrk, (uint8_t *)&Pwm1, 1);

    if(Vol_Max == 0xff)
    {
       Vol_Max = VolumeMAX;
       Vol_Min = VolumeMIN;
    }
    if(Pwm1 == 0xff)
    {
       Pwm1 = Vol_Max;
    }
    OCR1A = Pwm1;

    memset((void *)&LcdBuf[0],' ',32);

    FlagMute = 1;
    Modulation = WFM;
    FrcvWork   = 106600;
    rMEM();   // IndMEM = 0


      TIC32ini(0x44);
      TIC32clr(0xff);

      KeyRADIO();     sleep(100);

      Lm7001Set();
      TIC32clr(0x00);
      Lm7001Set();

    for (;;)
    {
      TmrYYY();

      key = GetKbd();

      if(key--)
      {
     uart_putchar('!');
     PutHEX(key);


	(* SwKey[((key) & 0x07)] )();

	LcdYes++;
      }

      ScanerS();

      RsControl();

      TIC32Pos(StrPos);

      Scope2Lcd();

      //TIC32Prn("-106.600");
    }
}

//////////////////
void FrcvUpDown(void)
{
    switch(direction)
    {
      case UP:   if(FrcvWork <  FrcvMax)
		 {
		    FrcvWork += FrcvStep;
		 }
		 else
		    direction = DOWN;

		 break;

      case DOWN: if(FrcvWork >  FrcvMin)
		 {
		    FrcvWork -= FrcvStep;
		 }
		 else
		    direction = UP;

		 break;
    }
    return;
}


//////////////////
void KeyPlus(void)
{
   if(contrast < 127)
   {
     uart_putchar('+');
     TIC32ini(contrast++);
     PutHEX(contrast);
   }
   if(Regim == RADIO)
   {
     direction = UP;

     FrcvUpDown();
     SetFrcv();
   }

   if(Regim == WR_MEM || Regim == RC_MEM)
   {
      if(++IndMEM > NUM_MEM-1)
	   IndMEM = 0;

      if(Regim == RC_MEM) rMEM();
   }

   if(Regim == SCAN_M && FlagBank == 0)
   {
      if(++ScanMBank > 3)
	   ScanMBank = 0;
   }

   if(Regim == SET_SQL) SQLup();
   if(Regim == SET_VOL) VolumeUp();
   if(Regim == UAPY   ) UapyUp();

   return;
}

//////////////////
void KeyMinus(void)
{
   if(contrast > 1)
   {
     uart_putchar('-');
     TIC32ini(contrast--);
     PutHEX(contrast);
   }


   if(Regim == RADIO)
   {
     direction = DOWN;

     FrcvUpDown();
     SetFrcv();
   }

   if(Regim == WR_MEM || Regim == RC_MEM)
   {
      if(--IndMEM < 0)
	   IndMEM = NUM_MEM-1;

      if(Regim == RC_MEM) rMEM();
   }

   if(Regim == SCAN_M && FlagBank == 0)
   {
      if(--ScanMBank < 0)
	   ScanMBank = 3;
   }

   if(Regim == SET_SQL) SQLdown();
   if(Regim == SET_VOL) VolumeDown();
   if(Regim == UAPY   ) UapyDown();

   return;
}


///////////////////////////////////////////
static uint8_t On2Lcd[3] = {' ','O','n'};
static uint8_t Of2Lcd[3] = {'O','f','f'};

/////////////////////////
void Flg2Lcd( uint8_t sw)
{
     memcpy((uint8_t *)&LcdBuf[6],(sw)? On2Lcd:Of2Lcd,3);
     return;
}

/////////////////////////////////////////////////////////
static uint8_t Gain2Lcd[4] = {'G','A','I','N'};
static uint8_t Uapy2Lcd[4] = {'U','a','p','y'};
static uint8_t Banc2Lcd[4] = {'B','a','n','c'};
static uint8_t Step2Lcd[4] = {'S','t','e','p'};
static uint8_t _sql2Lcd[4] = {'S','q','l',' '};
static uint8_t _vol2Lcd[4] = {'V','o','l',' '};


#define sGain 0
#define sUapy 1
#define sBanc 2
#define sStep 3
#define sSql  4
#define sVol  5

void Str2Lcd(uint8_t sw)
{
  uint8_t *p;

	if(sw == sGain) p = Gain2Lcd;
   else if(sw == sUapy) p = Uapy2Lcd;
   else if(sw == sBanc) p = Banc2Lcd;
   else if(sw == sStep) p = Step2Lcd;
   else if(sw == sSql ) p = _sql2Lcd;
   else                 p = _vol2Lcd;

   memset((uint8_t *)LcdBuf,' ',11);
   memcpy((uint8_t *)LcdBuf,  p, 4);
}


//////////////////
void Mod2Lcd(void)
{
  static uint8_t Mod4Lcd[3] = {'w','n','a'};

  LcdBuf[2] = Mod4Lcd[Modulation];
}

//////////////////
void KeyMOD(void)
{

   if(contrast < 127)
   {
     TIC32ini(contrast++);
   }


   if(Regim == RADIO)
   {
     if(++Modulation > AM)
	  Modulation = WFM;

     Mod2Lcd();
     Lm7001Set();
   }
   return;
}

//////////////////
void KeyRADIO(void)
{
     if(Regim == SET_VOL)
     {
	wBufEEPROM (ADR_EE_vWrk, (uint8_t *)&Pwm1, 1);
     }

     if(Regim == SCAN_F)
     {
	FlagScanF = 0;
	SetFmax();
	SetFmin();
     }
     FlagScanM = 0;
     Regim = RADIO;

     TimeClr = 0;

     LcdBuf[0] = 'P';
     LcdBuf[1] = ' ';
     LcdBlink  = 0;

     Mod2Lcd();
     SetFrcv();
}


//////////////////
void KeyGAIN(void)
{
     switch(Regim)
     {
       case GAIN:  TunerGain = (TunerGain)? 0:1;

       case RADIO:
		   Str2Lcd(sGain);
		   Flg2Lcd(TunerGain);

		   Regim = GAIN;
     }
     return;
}

/////////////////
void UapyUp(void)
{
     if(UapyFlg == 2)
     {
	if(Uapy < 255)
	   Uapy++;

	UapyOut();
     }
     return;
}
///////////////////
void UapyDown(void)
{
     if(UapyFlg == 2)
     {
	if(Uapy)
	   Uapy--;

	UapyOut();
     }
     return;
}

//////////////////
void KeyUapy(void)
{
     switch(Regim)
     {
       case UAPY:
			if(UapyFlg == 1)
			{
			   UapyFlg = 2;
			   Uapy = UapySet;
			}
		   else if(UapyFlg == 2)
			{
			   UapyFlg = 0;
			   UapySet = Uapy;
			   Uapy = 0xff;
			}
		   else {
			   UapyFlg = 1;
			}
			UapyOut();

       case RADIO:
		   Str2Lcd(sUapy);

		   if(UapyFlg != 2)
		   {
		      Flg2Lcd(UapyFlg);
		   }
		   Regim = UAPY;
     }
     return;
}

//////////////////
void KeyFinp(void)
{
     switch(Regim)
     {
       case FINPUT:
       case  RADIO:
		    Mod2Lcd();

		    memset((void *)&LcdBuf[Lcd4f],' ',7);

		    LcdBlink = _BV(Lcd4f);//0x0004;
		    LcdBuf[    0] = Lcd_F;
		    LcdBuf[Lcd4f] = Lcd__;

		    InpInd = 0;
		    Regim  = FINPUT;
		    break;

       case  WR_MEM:
		    cMEM();
		    LcdBlink = 0x01FF;
		    sleep(2000);
		    LcdBlink = 0x0003;
     }
     return;
}

////////////////////////
void Key_ii( uint8_t c )
{

/*
  uint32_t f;

	c += '0';

	LcdBlink <<= 1;

	LcdBuf[InpInd+Lcd4f] = c;

	InpBuf[InpInd++] = c;
	InpBuf[InpInd  ] = 0;

     if(InpInd >= InpBufLEN-1)
     {
	f = a2l((uint8_t *)InpBuf);

	SetFwork(f);
     }
     else
	LcdBuf[InpInd+Lcd4f] = '_';
*/
}

////////////////
void Key_0(void) { Key_ii(0); return; }
void Key_5(void) { Key_ii(5); return; }
void Key_6(void) { Key_ii(6); return; }
void Key_7(void) { Key_ii(7); return; }
void Key_8(void) { Key_ii(8); return; }
void Key_9(void) { Key_ii(9); return; }

void Key_3(void) { TIC32ContrastP(); return; }
void Key_4(void) { TIC32ContrastM(); return; }

////////////////
void Key_2(void)
{
   if(Cursor > 1)
   {
      TIC32Cur(127-Cursor,0);
      Cursor--;
      TIC32Cur(127-Cursor,1);

      if(Cursor < 40 && StrPos == 127-40)
      {
	TIC32Pos(StrPos);
	TIC32Prn("        ");
	StrPos = 0;
      }
   }
   return;
}

////////////////
void Key_1(void)
{
   if(Cursor < 127)
   {
      TIC32Cur(127-Cursor,0);
      Cursor++;
      TIC32Cur(127-Cursor,1);

      if(Cursor > 90 && StrPos == 0)
      {
	TIC32Pos(StrPos);
	TIC32Prn("        ");
	StrPos = 127-40;
      }
   }
   return;
}

///////////////////////////////////////
void KeySetHL(uint16_t addr, uint8_t c)
{
     switch(Regim)
     {
       case WR_H_L:
		    wBufEEPROM (addr, (uint8_t *)&FrcvWork, 4);

		    LcdBlink  = 0;
		    LcdBuf[0] = c;

		    sleep(2000);
		    KeyRADIO();
		    break;
       case  RADIO:
		    LcdBlink  = 0x0001;
		    LcdBuf[0] =   c;
		    LcdBuf[1] = ' ';

		    Regim  = WR_H_L;
     }
     return;
}

///////////////////
void KeySetHi(void) { KeySetHL(ADR_EE_Hi,'H'); return; }
void KeySetLo(void) { KeySetHL(ADR_EE_Lo,'L'); return; }

/////////////////////////////////////
uint8_t buff[8]; // for min size code
/////////////////////////////////////

////////////////
void wMEM(void)
{
     memcpy(buff,(uint8_t *)&FrcvWork,4);

     buff[3] = Modulation | 0xFC;

     wBufEEPROM (((uint16_t)IndMEM)<<2, buff, 4);
}

//////////////////
uint8_t rMEM(void)
{
     rBufEEPROM (((uint16_t)IndMEM)<<2, buff, 4);

     if((buff[3] & 0x03) != 0x03)
     {
	Modulation = buff[3] & 0x03;

	memcpy((uint8_t *)&FrcvWork,buff,3);
	Mod2Lcd();
	SetFrcv();
     }
     else
     {
	 memset((void *)&LcdBuf[Lcd4f],Lcd_Minus,7);
	 return 1;
     }
     return 0;
}

////////////////
void cMEM(void)
{
     buff[0] =
     buff[1] =
     buff[2] =
     buff[3] = 0xff;

     wBufEEPROM (((uint16_t)IndMEM)<<2, buff, 4);
}

///////////////////
void KeyWrMem(void)
{
     switch(Regim)
     {
       case WR_MEM:
		    wMEM();
		    LcdBlink = 0;

		    sleep(2000);
		    KeyRADIO();
		    break;
       case  RADIO:
		    LcdBlink = 0x0003;

		    Regim  = WR_MEM;
     }
     return;
}

///////////////////
void KeyRcMem(void)
{
     if(Regim == RADIO)
     {
	Regim = RC_MEM;

	rMEM();
     }
     return;
}

///////////////////
void KeyScanF(void)
{
     if(Regim == SCAN_F)
     {
	FlagScanF = (FlagScanF)? 0:1;
     }

     if(Regim == RADIO)
     {
	Regim = SCAN_F;

	LcdBuf[0] = Lcd_Poisk;  // "POISK"
	Mod2Lcd();

	rBufEEPROM (ADR_EE_Hi, (uint8_t *)&FrcvMax, 4);
	rBufEEPROM (ADR_EE_Lo, (uint8_t *)&FrcvMin, 4);

	if(FrcvMax < FrcvMin)
	{
	  rBufEEPROM (ADR_EE_Lo, (uint8_t *)&FrcvMax, 4);
	  rBufEEPROM (ADR_EE_Hi, (uint8_t *)&FrcvMin, 4);
	}
	if(FrcvMax & 0x80000000) SetFmax();
	if(FrcvMin & 0x80000000) SetFmin();

	if( FrcvWork > FrcvMax ||
	    FrcvWork < FrcvMin || FlagBegin)
	{
	    FrcvWork = FrcvMin;
	    FlagBegin= 0;
	    SetFrcv();
	}
	FlagScanF = 1;
     }
     return;
}

///////////////////
void KeyScanM(void)
{
     if(Regim == SCAN_M)
     {
	if(FlagBank)
	{
	   FlagScanM = (FlagScanM)? 0:1;
	}
	else
	{
	   FlagScanM = 1;
	   FlagBank  = 1;
	}
     }

     if(Regim == RADIO)
     {
	Regim = SCAN_M;

	Str2Lcd(sBanc);
	FlagBank = 0;
     }
     return;
}

  /////////////////////
  /////////////////////
  volatile uint8_t SQL;

////////////////
void SQLup(void)
{
     if(SQL < 255)
	SQL++;

     TimeClr = AUTO_CLR;
}

//////////////////
void SQLdown(void)
{
     if(SQL)
	SQL--;

     TimeClr = AUTO_CLR;
}

///////////////////
void KeySQL(void)
{
     if(Regim == RADIO)
     {
	Regim = SET_SQL;

	Str2Lcd(sSql);
	TimeClr = AUTO_CLR;
     }
     return;
}

///////////////////
void PduRADIO(void)
{
   if( Regim != SET_VOL &&
       Regim != SET_SQL &&
       Regim != RADIO   )
   {
       KeyRADIO();
   }
   return;
}

void RsControl(void)
{
  uint8_t inp;
  uint8_t   i;
  uint32_t  f;

  if(Regim == RADIO || Regim == SCAN_F)
  {
     if(RxInd != TxInd)
     {
	if((inp = RxInpBuf[TxInd++]) != '\n' && EchoYes)
	{
	   uart_putchar(inp);
	}
	//uart_putchar(inp = RxInpBuf[TxInd++]);

	if(RxInd >= RxInpBufLEN)
	{
	   RcInd = 0;
	   RxInd = 0;
	   TxInd = 0;
	   uart_putstr(ERROR);

	   return;
	}
	if(isdigit(inp))
	{
	   RcInpBuf[RcInd++] = inp;
	}

	if(inp == '\r')
	{
	   RcInpBuf[RcInd] = '\0';
	   RcInd = 0;

	   if(Regim == SCAN_F)
	   {
	      RxInpBuf[0] = 0;
	      KeyRADIO();
	   }
	   f = a2l((uint8_t *)RcInpBuf);

	   i = (uint8_t)(f & 0x000000FF);

	   // use uint8_t inp like tmp

	   inp = (RxInpBuf[0] == '\n')? 1:0;

	   switch(RxInpBuf[inp])
	   {
	    case '+': KeyPlus (); break;
	    case '-': KeyMinus(); break;

	    case 'M':
		      if((Modulation = i & 0x03) == 3)
			  Modulation = 0;
		      Mod2Lcd();
		      Lm7001Set();
		      break;

	    case 'G': TunerGain = i & 0x01;
		      break;

	    case 'S': FindStep = i &0x07;
		      FrcvStep = Step[FindStep];
		      break;

	    case 'U': Uapy    = i;
		      UapyFlg = 2; UapyOut();
		      break;

	    case 'E': EchoYes = i;
		      break;

	    case 'L':
		      if(i == 2)
		      {
			 FlagSQL = 1;
		      }
		      else
		      {
			 FlagMute = i & 0x01;
			 FlagSQL = 0;
			 Lm7001Set();
		      }
		      break;

	    case '.': fWorkPrn();
		      break;

	    case ',': break;

	    case 'D': if(i)
		      {
			LcdWork = 1;
		      }
		      else
		      {
			/*
			LcdWork = 0;
			cli();
			LcdOut2(0);
			LcdOut2(0);
			sei();
			*/
		      }
		      break;

	    case 'H': wBufEEPROM (ADR_EE_Hi, (uint8_t *)&f, 4);
		      break;

	    case 'B': wBufEEPROM (ADR_EE_Lo, (uint8_t *)&f, 4);
		      break;


	    case 'F': SetFwork(f);
		      break;

	    case 'P': KeyScanF();
		      break;

	    case 'N': FlagBegin++;
		      KeyScanF();
		      break;

	    case 'V': wBufEEPROM (ADR_EE_vMax, (uint8_t *)&i, 1);
		      wBufEEPROM (ADR_EE_vWrk, (uint8_t *)&i, 1); break;
	    case 'v': wBufEEPROM (ADR_EE_vMin, (uint8_t *)&i, 1); break;

	   }
	   RxInd = 0;
	   TxInd = 0;
	   uart_putstr(PROMPT);
	}
     }
  }
  return;
}


///////////////////
void VolumeUp(void)
{
   if(Pwm1 > Vol_Min)
   {
     OCR1A = Pwm1--;
   }
   TimeClr = AUTO_CLR;
   return;
}

/////////////////////
void VolumeDown(void)
{
   if(Pwm1 < Vol_Max)
   {
     OCR1A = Pwm1++;
   }
   TimeClr = AUTO_CLR;
   return;
}

///////////////////
void KeyVOL(void)
{
     if(Regim == RADIO)
     {
	Regim = SET_VOL;

	Str2Lcd(sVol);
	TimeClr = AUTO_CLR;
     }
     return;
}

//////////////////////////////////////
//////////////////////////////////////

	  uint8_t l2aBuff[8];

//////////////////////////////////////
uint8_t * ul2a(uint32_t c,uint8_t len)
{
  uint8_t  i;
  uint8_t *p;

   p = &l2aBuff[len];
  *p = 0;

  for(i = len; i > 0; i--)
  {
      p--;
     *p  = c%10 +'0';
      c /= 10;
  }
  return p;
}

////////////////////////
uint32_t a2l(uint8_t *p)
{
  uint32_t l = 0;

  while(*p)
  {
     l *= 10;
     l += *p-'0';
     p++;
  }
  return l;
}

//////////////////////////////////////
//                                  //
//            TIC-32                //
//                                  //
//////////////////////////////////////

#define TIC32_ADR  0x78
#define TIC32_CON1 0x80
#define TIC32_CON2 0x40

#define PAGE_00 0x01
#define PAGE_01 0x09
#define PAGE_10 0x0A

#define VERTICAL   0x02
#define HORISONTAL 0x00



////////////////////
void TIC32ini(uint8_t voltage)
{
     if(voltage > 127) voltage = 127;

     I2CStart();
     I2COutByte(TIC32_ADR);

     I2COutByte(TIC32_CON1); I2COutByte(PAGE_00);
     I2COutByte(TIC32_CON1); I2COutByte(0x22);    // ON | Vertical Addressing
     I2COutByte(TIC32_CON1); I2COutByte(PAGE_01);
     I2COutByte(TIC32_CON1); I2COutByte(0x0C);    // D  =1  IM =0  E  =0
     I2COutByte(TIC32_CON1); I2COutByte(0x05);    // M1 =0  M0 =1
     I2COutByte(TIC32_CON1); I2COutByte(0x14);    // BS2=1  BS1=0  BS0=0


     I2COutByte(TIC32_CON1); I2COutByte(PAGE_00);
     I2COutByte(TIC32_CON1); I2COutByte(PAGE_10);
     I2COutByte(TIC32_CON1); I2COutByte(0x0A);    // S1 =1 S0 =0
     I2COutByte(TIC32_CON1); I2COutByte(0x20);    // TC2=0 TC1=0 TC0=0
     I2COutByte(TIC32_CON1); I2COutByte(0x05);    // PRS=0 HVE=1
     I2COutByte(TIC32_CON1); I2COutByte(0x80|voltage);


     I2COutByte(TIC32_CON1); I2COutByte(PAGE_00);
     I2COutByte(TIC32_CON1); I2COutByte(0x40);     // Set Y=0
     I2COutByte(TIC32_CON1); I2COutByte(0x80);     // Set X=0

     I2CStop();
     return;
}


////////////////////////
void TIC32clr(uint8_t c)
{
  uint16_t   i;

     I2CStart();

     I2COutByte(TIC32_ADR);
     I2COutByte(TIC32_CON2);

     for(i=0; i<640; i++)
     {
       I2COutByte(c);
     }
     I2CStop();
     return;
}



////////////////////////
void TIC32Pos(uint8_t x)
{
     I2CStart();

     I2COutByte(TIC32_ADR);
     I2COutByte(TIC32_CON1); I2COutByte(PAGE_00);
     I2COutByte(TIC32_CON1); I2COutByte(0x40);     // Set Y=0
     I2COutByte(TIC32_CON1); I2COutByte(0x80|x);   // Set X=x

     I2CStop();
}

///////////////////////////////
void TIC32Addressing(uint8_t v)
{
     I2CStart();

     I2COutByte(TIC32_ADR);

     I2COutByte(TIC32_CON1); I2COutByte(PAGE_00);
     I2COutByte(TIC32_CON1); I2COutByte(0x20|v);

     I2CStop();
}


////////////////////
void TIC32Line(uint8_t *p)
{
  uint8_t i;

     TIC32Addressing(VERTICAL);

     I2CStart();

     I2COutByte(TIC32_ADR);
     I2COutByte(TIC32_CON2);

     for(i=0; i<4; i++)
     {
       I2COutByte(*p++);
     }
     I2COutByte(0);
     I2CStop();
     return;
}

/////////////////////////
void TIC32ContrastP(void)
{
   if(contrast < 127)
   {
     TIC32ini(contrast++);
   }
}

/////////////////////////
void TIC32ContrastM(void)
{
   if(contrast > 1)
   {
     TIC32ini(contrast--);
   }
}

////////////////////
void ScopeMeter(void)
{
  uint8_t  i;

  uint16_t level = 0;

    for(i=0; i < 6; i++)
    {
      level += LoBYTE(GetADC(CH_SMETR)>>2);
    }
    level = (level/i)>>3;

    Vline(31-(uint8_t)level,31);

    //Dscope = Iscope;

    //TIC32Pos (127-Iscope);
    //TIC32Line(disp);

    return;
}

///////////////////////////
uint8_t *GetFont(uint8_t c)
{
  static char tab[52]={0x00,0x00,0x00,0x00, // ' '
		       0xfe,0x82,0x82,0xfe, // '0'
		       0x00,0x00,0x00,0xfe, // '1'
		       0xf2,0x92,0x92,0x9e, // '2'
		       0x92,0x92,0x92,0xfe, // '3'
		       0x1e,0x10,0x10,0xfe, // '4'
		       0x9e,0x92,0x92,0xf2, // '5'
		       0xfe,0x92,0x92,0xf2, // '6'
		       0x02,0x02,0x02,0xfe, // '7'
		       0xfe,0x92,0x92,0xfe, // '8'
		       0x9e,0x92,0x92,0xfe, // '9'
		       0x00,0xc0,0xc0,0x00, // '.'
		       0x10,0x10,0x10,0x10  // '-'
		      };

  if(c == '.') c='9'+1;
  if(c == '-') c='9'+2;

  if(c >= '0' && c <= ('9'+2))
  {
     return(tab+((c-'0'+1)<<2));
  }
  return tab;
}

/////////////////////////
void TIC32Prn(uint8_t *p)
{
  uint8_t *f;
  uint8_t  i;

  TIC32Addressing(HORISONTAL);

  I2CStart();

  I2COutByte(TIC32_ADR);
  I2COutByte(TIC32_CON2);

  while(*p)
  {
     f = GetFont(*p++);

     for(i=0; i<4; i++)
     {
       I2COutByte(*f++);
     }
     I2COutByte(0);
   }
   I2CStop();
   return;
}

///////////////////////////////////
void TIC32Cur(uint8_t x,uint8_t sw)
{
  if(x)
  {
    TIC32Pos(x-1);
  }
  else
    TIC32Pos(0);

  TIC32Addressing(HORISONTAL);

  I2CStart();

  I2COutByte(TIC32_ADR);
  I2COutByte(TIC32_CON2);

  if(sw)
  {
     if(x)
       I2COutByte(0x01);

       I2COutByte(0x07);

     if(x < 127)
       I2COutByte(0x01);
  }
  else
  {
     if(x)
       I2COutByte(0x00);

       I2COutByte(0x00);

     if(x < 127)
       I2COutByte(0x00);
  }

  I2CStop();
  return;
}


//////////////////
void Scope2Lcd(void)
{
  char *t;
  char *p;
  int16_t f;
  uint8_t i;

    t = cccBuf;
    f = 10*((int16_t)Cursor-WHIDE/2);

    *t = '-';

    if(f <= 0)
    {
       f *= -1;
       *t = ' ';
    }
    t++;

    p = ul2a(f,4);

    for(i=0; i < 4; i++)
    {
      if(i == 1) *t++ = '.';

      *t++ = *p++;
    }
    *t = '\0';

    TIC32Prn(cccBuf);
    return;
}

/*
//////////////////
void Scope2Lcd(void)
{
  char *p;

  uint8_t i;
  uint8_t flag = 0;

    Fscope = (RCVPCH1-RCVPCH2)+10*((int16_t)Cursor-WHIDE/2);

    p = ul2a(Fscope,6);

    for(i=0; i < 6; i++)
    {
      cccBuf[i]=*p;

      if(*p != '0') flag++;

	       LcdBuf[i+Lcd4f] &= Lcd_Comma;
      if(flag) LcdBuf[i+Lcd4f] |= LcdDigit(*p-'0');

      p++;
    }
    return;
}
*/
